package vip.xumy.drools.conf;

import java.io.File;
import java.io.IOException;

import org.kie.api.KieBase;
import org.kie.api.KieBaseConfiguration;
import org.kie.api.KieServices;
import org.kie.api.builder.KieBuilder;
import org.kie.api.builder.KieFileSystem;
import org.kie.api.builder.KieRepository;
import org.kie.api.builder.ReleaseId;
import org.kie.api.conf.EventProcessingOption;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.KieSessionConfiguration;
import org.kie.api.runtime.conf.ClockTypeOption;
import org.kie.internal.conf.MultithreadEvaluationOption;
import org.kie.internal.io.ResourceFactory;
import org.springframework.boot.system.ApplicationHome;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.stereotype.Component;

@Component
@Configuration
public class DroolsConfig {
	private static final String RULES_PATH = "rules/";
	public static final String CLASSPATH = "classpath:";
	public static final String JAR_ROOT_PATH;
	private static KieContainer kieContainer;
	private static KieBase kieBase;

	static {
		ApplicationHome h = new ApplicationHome(DroolsConfig.class);
		File jarF = h.getSource();
		JAR_ROOT_PATH = jarF.getParentFile().getAbsolutePath() + '/';
	}

	public void reloadRules() throws IOException {
		if (kieContainer != null) {
			kieContainer.dispose();
			kieContainer = null;
			kieBase = null;
		}
		kieBase();
	}

	private KieFileSystem kieFileSystem() throws IOException {
		KieFileSystem kieFileSystem = getKieServices().newKieFileSystem();
		File dir = new File(JAR_ROOT_PATH + RULES_PATH);
		if (dir.exists()) {
			for (File file : dir.listFiles()) {
				kieFileSystem.write(ResourceFactory.newFileResource(file));
			}
		} else {
			ResourcePatternResolver resourcePatternResolver = new PathMatchingResourcePatternResolver();
			Resource[] resources = resourcePatternResolver.getResources(CLASSPATH + RULES_PATH + "**/*.*");
			for (Resource resource : resources) {
				kieFileSystem.write(ResourceFactory.newFileResource(resource.getFile()));
			}
		}
		return kieFileSystem;
	}

	private ReleaseId build() throws IOException {
		final KieRepository kieRepository = getKieServices().getRepository();
		// 设置时间格式
		System.setProperty("drools.dateformat", "yyyy-MM-dd HH:mm:ss");
		ReleaseId releaseId = kieRepository.getDefaultReleaseId();
		kieRepository.removeKieModule(releaseId);
		kieRepository.addKieModule(kieRepository::getDefaultReleaseId);
		KieBuilder kieBuilder = getKieServices().newKieBuilder(kieFileSystem());
		kieBuilder.buildAll();
		return releaseId;
	}

//	private static final String JAVA_ROOT = "src/main/java/";
//	private static final String RESOURCES_ROOT = "src/main/resources/";
//	private String getPath(org.kie.api.io.Resource resource) {
//		String target = resource.getTargetPath() != null ? resource.getTargetPath() : resource.getSourcePath();
//		if (target != null) {
//			String prefix = resource.getResourceType() == ResourceType.JAVA ? JAVA_ROOT : RESOURCES_ROOT;
//			int prefixPos = target.indexOf(prefix);
//			String path = prefixPos >= 0 ? target.substring(prefixPos) : prefix + target;
//			path = path.replace(java.io.File.separatorChar, '/');
//			return path;
//		} else {
//			throw new RuntimeException(
//					"Resource does not have neither a source nor a target path. Impossible to add it to the bundle. Please set either the source or target name of the resource before adding it."
//							+ resource.toString());
//		}
//	}

	public KieContainer kieContainer() throws IOException {
		if (kieContainer != null) {
			return kieContainer;
		}
		kieContainer = getKieServices().newKieContainer(build());
		return kieContainer;
	}

	private KieServices getKieServices() {
		return KieServices.Factory.get();
	}

	public KieBase kieBase() throws IOException {
		if (kieBase != null) {
			return kieBase;
		}
		KieBaseConfiguration baseConf = KieServices.Factory.get().newKieBaseConfiguration();
		baseConf.setOption(EventProcessingOption.STREAM);
		baseConf.setOption(MultithreadEvaluationOption.YES);
		kieBase = kieContainer().newKieBase(baseConf);
		return kieBase;
	}

	public KieSession kieSession() throws IOException {
		KieSessionConfiguration sessConf = KieServices.Factory.get().newKieSessionConfiguration();
		sessConf.setOption(ClockTypeOption.get("pseudo"));
		KieSession kieSession = kieBase().newKieSession(sessConf, null);
		return kieSession;
	}

}
